namespace dnnl {
namespace impl {
namespace gpu {
namespace intel {
namespace ocl {
const char *atomic_reduction_kernel = R"==(/*******************************************************************************)==""\n"
R"==(* Copyright 2021-2024 Intel Corporation)==""\n"
R"==(*)==""\n"
R"==(* Licensed under the Apache License, Version 2.0 (the "License");)==""\n"
R"==(* you may not use this file except in compliance with the License.)==""\n"
R"==(* You may obtain a copy of the License at)==""\n"
R"==(*)==""\n"
R"==(* http:)==""\n"
R"==(*)==""\n"
R"==(* Unless required by applicable law or agreed to in writing, software)==""\n"
R"==(* distributed under the License is distributed on an "AS IS" BASIS,)==""\n"
R"==(* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.)==""\n"
R"==(* See the License for the specific language governing permissions and)==""\n"
R"==(* limitations under the License.)==""\n"
R"==(*******************************************************************************/)==""\n"
R"==(#include "gpu/intel/ocl/dispatch.h")==""\n"
R"==(#include "gpu/intel/ocl/ocl_types.h")==""\n"
R"==(#include "gpu/intel/ocl/reduction/ocl_reduction.h")==""\n"
R"==(#include "gpu/intel/ocl/types_interop.h")==""\n"
R"==(#if defined(IS_MAX))==""\n"
R"==(#define ATOMIC_ACCUMULATE(atomic_p, data) atomic_max_global(atomic_p, data))==""\n"
R"==(#elif defined(IS_MIN))==""\n"
R"==(#define ATOMIC_ACCUMULATE(atomic_p, data) atomic_min_global(atomic_p, data))==""\n"
R"==(#elif defined(IS_MEAN) || defined(IS_SUM))==""\n"
R"==(#define ATOMIC_ACCUMULATE(atomic_p, data) atomic_add_global(atomic_p, data))==""\n"
R"==(#endif)==""\n"
R"==(#define DEF_atomic_accumulate(dt) \)==""\n"
R"==(dt atomic_accumulate(int alg, __global ATOMIC(dt) * atomic_p, dt data) { \)==""\n"
R"==(switch (alg) { \)==""\n"
R"==(case (REDUCTION_MAX): return atomic_max_global(atomic_p, data); \)==""\n"
R"==(case (REDUCTION_MIN): return atomic_min_global(atomic_p, data); \)==""\n"
R"==(case (REDUCTION_MEAN): \)==""\n"
R"==(case (REDUCTION_SUM): return atomic_add_global(atomic_p, data); \)==""\n"
R"==(} \)==""\n"
R"==(printf("Atomic accumulate on unexpected algorithm\n"); \)==""\n"
R"==(return 0; \)==""\n"
R"==(})==""\n"
R"==(#if ATOMIC_REDUCTION_SIZE > 1)==""\n"
R"==(#define MAYBE_ATOMIC(x) ATOMIC(x))==""\n"
R"==(DEF_atomic_accumulate(float);)==""\n"
R"==(#else)==""\n"
R"==(#define MAYBE_ATOMIC(x) x)==""\n"
R"==(#endif)==""\n"
R"==(#define BLOCK_READ_DATA_T(data_ptr) \)==""\n"
R"==(AS_VECT_DATA_T(VECT_BLOCK_READ((const __global BLOCK_DATA_T *)data_ptr)))==""\n"
R"==(#if VECT_DT_N == 1)==""\n"
R"==(#define GET_ELEM(x, idx) x)==""\n"
R"==(#else)==""\n"
R"==(#define GET_ELEM(x, idx) x[idx])==""\n"
R"==(#endif)==""\n"
R"==(#if VECT_DT_N == 1)==""\n"
R"==(#define TO_VECT_DST TO_DST)==""\n"
R"==(#define VECT_DST_DATA_T DST_DATA_T)==""\n"
R"==(#define VECT_DEF_ACC_TO_FLOAT convert_float)==""\n"
R"==(#else)==""\n"
R"==(#define VECT_DST_DATA_T CONCAT2(DST_DATA_T, VECT_DT_N))==""\n"
R"==(#define VECT_DEF_ACC_TO_FLOAT CONCAT2(convert_float, VECT_DT_N))==""\n"
R"==(#if VECT_DT_N == 2)==""\n"
R"==(#define TO_VECT_DST TO_DST2)==""\n"
R"==(#elif VECT_DT_N == 4)==""\n"
R"==(#define TO_VECT_DST TO_DST4)==""\n"
R"==(#elif VECT_DT_N == 8)==""\n"
R"==(#define TO_VECT_DST TO_DST8)==""\n"
R"==(#endif)==""\n"
R"==(#endif)==""\n"
R"==(#define REDUCTION_WI_COUNT (ATOMIC_REDUCTION_SIZE * LOCAL_SIZE))==""\n"
R"==(KERNEL_ATTR)==""\n"
R"==(__kernel void atomic_reduce(__global SRC_DATA_T *src,)==""\n"
R"==(__global MAYBE_ATOMIC(DST_DATA_T) * dst, int inner_size, off_t div,)==""\n"
R"==(float power, float eps, off_t num_reductions,)==""\n"
R"==(dispatch_gws_rt_params_t gws_params) {)==""\n"
R"==(ASSUME(inner_size > 0);)==""\n"
R"==(ASSUME(num_reductions > 0);)==""\n"
R"==(const int local_idx = get_sub_group_id();)==""\n"
R"==(const int sglid = get_sub_group_local_id();)==""\n"
R"==(const int subgroup_size = get_max_sub_group_size();)==""\n"
R"==(off_t atomic_idx = GWS_GET_OFF(ATOMIC, gws_params);)==""\n"
R"==(off_t SRC_OFF = GWS_GET_OFF(SRC, gws_params) - sglid;)==""\n"
R"==(off_t src_ig_idx = SRC_OFF / subgroup_size;)==""\n"
R"==(SRC_OFF += src_ig_idx * (VECT_DT_N - 1) * subgroup_size;)==""\n"
R"==(src += SRC_OFF;)==""\n"
R"==(off_t DST_OFF = GWS_GET_OFF(DST, gws_params);)==""\n"
R"==(off_t dst_ig_idx = DST_OFF / subgroup_size;)==""\n"
R"==(DST_OFF += dst_ig_idx * (VECT_DT_N - 1) * subgroup_size;)==""\n"
R"==(dst += DST_OFF;)==""\n"
R"==(const int beg = local_idx + atomic_idx * LOCAL_SIZE;)==""\n"
R"==(ASSUME(beg < REDUCTION_WI_COUNT);)==""\n"
R"==(const int tail_count = num_reductions % REDUCTION_WI_COUNT;)==""\n"
R"==(VECT_DEF_ACC_DATA_T acc;)==""\n"
R"==(init_acc(REDUCTION_ALG, &acc);)==""\n"
R"==(const int iters = div_up(num_reductions - beg, REDUCTION_WI_COUNT);)==""\n"
R"==(if (beg < tail_count) {)==""\n"
R"==(unroll_for_by(FULL_UNROLL_FACTOR)(off_t i = 0; i < iters; i++) {)==""\n"
R"==(const off_t src_off = (beg + i * REDUCTION_WI_COUNT) * inner_size;)==""\n"
R"==(const VECT_DATA_T src_val = BLOCK_READ_DATA_T(&src[src_off]);)==""\n"
R"==(unroll_for(uint i = 0; i < VECT_DT_N; i++) {)==""\n"
R"==(GET_ELEM(acc, i) = reduce(REDUCTION_ALG, GET_ELEM(acc, i),)==""\n"
R"==(TO_DEF_ACC_DATA_T(GET_ELEM(src_val, i)), power);)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(} else {)==""\n"
R"==(unroll_for_by(TAIL_UNROLL_FACTOR)(off_t i = 0; i < iters; i++) {)==""\n"
R"==(const off_t src_off = (beg + i * REDUCTION_WI_COUNT) * inner_size;)==""\n"
R"==(const VECT_DATA_T src_val = BLOCK_READ_DATA_T(&src[src_off]);)==""\n"
R"==(unroll_for(uint i = 0; i < VECT_DT_N; i++) {)==""\n"
R"==(GET_ELEM(acc, i) = reduce(REDUCTION_ALG, GET_ELEM(acc, i),)==""\n"
R"==(TO_DEF_ACC_DATA_T(GET_ELEM(src_val, i)), power);)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(__local DEF_ACC_DATA_T)==""\n"
R"==(local_acc_buf[LOCAL_SIZE][GWS_SGS_DEFAULT * VECT_DT_N];)==""\n"
R"==(unroll_for(int i = 0; i < VECT_DT_N; i++) {)==""\n"
R"==(local_acc_buf[local_idx][sglid + i * subgroup_size] = GET_ELEM(acc, i);)==""\n"
R"==(})==""\n"
R"==(barrier(CLK_LOCAL_MEM_FENCE);)==""\n"
R"==(if (local_idx == 0) {)==""\n"
R"==(VECT_DEF_ACC_DATA_T local_acc;)==""\n"
R"==(init_acc(SECONDARY_REDUCTION_ALG, &local_acc);)==""\n"
R"==(unroll_for(int slm_off = 0; slm_off < LOCAL_SIZE; slm_off++) {)==""\n"
R"==(unroll_for(int v = 0; v < VECT_DT_N; v++) {)==""\n"
R"==(DEF_ACC_DATA_T slm_data)==""\n"
R"==(= local_acc_buf[slm_off][sglid + v * subgroup_size];)==""\n"
R"==(GET_ELEM(local_acc, v) = reduce(SECONDARY_REDUCTION_ALG,)==""\n"
R"==(GET_ELEM(local_acc, v), slm_data, power);)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(#if VECT_DT_N == 1)==""\n"
R"==(float f = finalize(REDUCTION_ALG, convert_float(GET_ELEM(local_acc, i)),)==""\n"
R"==(div, power, eps);)==""\n"
R"==(DST_DATA_T vect_dst_data = TO_DST(f);)==""\n"
R"==(#else)==""\n"
R"==(VECT_DST_DATA_T vect_dst_data;)==""\n"
R"==(unroll_for(uint i = 0; i < VECT_DT_N; i++) {)==""\n"
R"==(float f = finalize(REDUCTION_ALG,)==""\n"
R"==(convert_float(GET_ELEM(local_acc, i)), div, power, eps);)==""\n"
R"==(GET_ELEM(vect_dst_data, i) = TO_DST(f);)==""\n"
R"==(})==""\n"
R"==(#endif)==""\n"
R"==(unroll_for(int v = 0; v < VECT_DT_N; v++) {)==""\n"
R"==(DST_DATA_T dst_data = GET_ELEM(vect_dst_data, v);)==""\n"
R"==(#if ATOMIC_REDUCTION_SIZE > 1)==""\n"
R"==(DST_DATA_T old_val = atomic_accumulate()==""\n"
R"==(REDUCTION_ALG, &dst[v * subgroup_size], dst_data);)==""\n"
R"==(#else)==""\n"
R"==(dst[v * subgroup_size] = dst_data;)==""\n"
R"==(#endif)==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==(})==""\n"
R"==()==";
}
}
}
}
}